<?php
declare(strict_types=1);

namespace Inti\Provider\Service;

use Inti\Provider\ProviderInterface;
use Inti\Provider\Type\TypeInterface;

/**
 * Interface ProviderManagerInterface
 *
 * @package Inti\Provider\Service
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-provider
 */
interface ProviderManagerInterface
{
    /**
     * @param string $type
     * @param TypeInterface $providedType
     * @return bool
     */
    public function canProvide(string $type, TypeInterface $providedType): bool;

    /**
     * @param string $type
     * @param TypeInterface $providedType
     * @return ProviderInterface|null
     */
    public function getProvider(string $type, TypeInterface $providedType): ?ProviderInterface;

    /**
     * @param string|null $type
     * @param TypeInterface|null $providedType
     * @return ProviderInterface[]
     */
    public function getProviders(?string $type = null, ?TypeInterface $providedType = null): array;

    public function registerProvider($provider): void;
    public function registerProviders(array $providers): void;
}
