<?php
declare(strict_types=1);

namespace IntiTest\Provider;

use Inti\Provider\Query\QueryInterface;
use Inti\Provider\Query\QueryTrait;

use function array_combine;
use function array_map;
use function array_slice;
use function count;
use function in_array;
use function preg_match;
use function preg_quote;

/**
 * Class ArrayProvider
 *
 * @package IntiTest\Provider
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-provider
 */
class ArrayProvider implements QueryInterface
{
    use QueryTrait;

    protected array $fields;
    protected array $items;

    public function __construct(array $fields, array $items)
    {
        $this->fields = $fields;
        $this->items = $items;
    }

    public function getCount(): int
    {
        return count($this->getItems());
    }

    public function getItem(): ?array
    {
        return $this->getItems()[0] ?? null;
    }

    public function getItems(): array
    {
        $items = [];

        foreach ($this->items as $item) {
            if (isset($this->ids) && !empty($this->ids)) {
                if (in_array($item[0], $this->ids)) {
                    $items[] = $item;
                }
            } else if (isset($this->slugs) && !empty($this->slugs)) {
                if (in_array($item[2], $this->slugs)) {
                    $items[] = $item;
                }
            } else if (isset($this->search) && !empty($this->search)) {
                if (preg_match('/' . preg_quote($this->search, '/') . '/', $item[1])) {
                    $items[] = $item;
                }
            } else {
                $items[] = $item;
            }
        }

        if ($this->offset) {
            $items = array_slice($items, $this->offset);
        }

        if ($this->limit) {
            $items = array_slice($items, 0, $this->limit);
        }

        return array_map(function (array $item) {
            return array_combine($this->fields, $item);
        }, $items);
    }
}
