<?php
declare(strict_types=1);

namespace IntiTest\Provider;

use Inti\Provider\Query\QueryInterface;
use PHPUnit\Framework\TestCase;
use function array_combine;
use function array_map;

/**
 * Class ArrayProviderTest
 *
 * @package IntiTest\Provider
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-provider
 */
final class ArrayProviderTest extends TestCase
{
    protected QueryInterface $provider;

    private array $data = [
        [1, 'First item', 'first-item'],
        [2, 'Second item', 'second-item'],
        [3, 'Third item', 'third-item'],
    ];

    private array $expected;

    private array $fields = ['id', 'title', 'slug'];

    protected function setUp(): void
    {
        $this->provider = new ArrayProvider($this->fields, $this->data);
        $this->expected = array_map(function (array $item) {
            return array_combine($this->fields, $item);
        }, $this->data);
    }

    public function testCanGetItemById()
    {
        $item = $this->provider->query()
            ->id(2)
            ->getItem();

        $this->assertEquals($this->expected[1], $item);
    }

    public function testCanGetItemsByIds()
    {
        $items = $this->provider->query()
            ->ids([1, 3])
            ->getItems();

        $this->assertEquals([$this->expected[0], $this->expected[2]], $items);
    }

    public function testCanLimitResults()
    {
        $items = $this->provider->query()
            ->limit(2)
            ->getItems();

        $this->assertEquals([$this->expected[0], $this->expected[1]], $items);
    }

    public function testCanOffsetResults()
    {
        $items = $this->provider->query()
            ->limit(1)
            ->offset(2)
            ->getItems();

        $this->assertEquals($this->expected[2], $items[0]);
    }

    public function testCanSearchResults()
    {
        $items = $this->provider->query()
            ->search('Second')
            ->getItems();

        $this->assertEquals([$this->expected[1]], $items);
    }

    public function testCanGetItemBySlug()
    {
        $item = $this->provider->query()
            ->slug('second-item')
            ->getItem();

        $this->assertEquals($this->expected[1], $item);
    }

    public function testCanGetItemsBySlugs()
    {
        $items = $this->provider->query()
            ->slugs(['first-item', 'third-item'])
            ->getItems();

        $this->assertEquals([$this->expected[0], $this->expected[2]], $items);
    }
}
